import sharp from 'sharp';
import { bytesToKbytes, getExt, getFileName, streamToBuffer } from './utils';
import bent from 'bent';
export class Driver {
    constructor({ name }) {
        this.name = name;
        return new Proxy(this, {
            get: (target, prop) => {
                if (typeof target[prop] !== 'function' || target[prop].name === 'url') {
                    return target[prop];
                }
                return async (...args) => {
                    try {
                        return await target[prop](...args);
                    }
                    catch (error) {
                        if (!this.errorHandler) {
                            throw error;
                        }
                        this.errorHandler(error);
                    }
                };
            },
        });
    }
    /**
     * Upload image from specific URI and store it into `filename`
     *
     * @param uri URI of image
     * @param path Filename included location to store image.
     * @param ignoreHeaderContentType ignore checking content-type header.
     * @returns Promise<any>
     */
    async uploadImageFromExternalUri(uri, path, ignoreHeaderContentType = false) {
        const head = bent('HEAD');
        const get = bent('GET', 200, 'buffer');
        const res = await head(uri);
        if (ignoreHeaderContentType ||
            (res.headers['content-type'] && res.headers['content-type'].includes('image'))) {
            const imageBuffer = await get(uri);
            return this.put(imageBuffer, path);
        }
        throw new Error('Not an image: ' + uri);
    }
    async imageStats(path, keepBuffer = false) {
        const stream = await this.get(path);
        const buffer = await streamToBuffer(stream);
        const { format, size, width, height } = await sharp(buffer).metadata();
        return {
            name: getFileName(path),
            path,
            size: bytesToKbytes(size),
            width,
            height,
            mime: format,
            ext: getExt(path),
            hash: null,
            ...(keepBuffer && { buffer }),
        };
    }
}
